"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Expands a pattern to all matching conrete paths over an object
 * @param pattern the pattern to expand
 * @param object the object to expand onto
 *
 * @example
 * ```javascript
 * const o = {
 *   foo: {
 *     bar: "bar",
 *     baz: "baz",
 *   },
 *   foo2: [
 *     "one",
 *     "two",
 *   ]
 * }
 * let result = expand(["*"], o)
 * // result === [ ["foo"], ["foo2"] ]
 * let result = expand(["*", "*"], o)
 * // result === [ ["foo", "bar"], ["foo", "baz"], ["foo2", 0], ["foo2", 1] ]
 * let result = expand(["foo", "*"], o)
 * // result ===  [ ["foo", "bar"], ["foo", "baz"] ]
 * ```
 */
// TODO: support "**" path segment
function expand(pattern, object) {
    var patternArr = new Array().concat(pattern);
    var segment = patternArr[0];
    // primitive type does return theirself, if the match the current segment
    if (null === object || typeof object !== "object") {
        return []; // matches(segment, object) ? [object] : []
    }
    else {
        if (patternArr.length > 1) {
            // find all subobject that matches segment
            // and return all results of expand(patternArr.slice(1), subObj).flatMap().map(path => [segment, ...path])
            if (object instanceof Array) {
                return object
                    .filter(function (_, i) { return matches(segment, i); })
                    .map(function (element, i) {
                    return expand(patternArr.slice(1), element).map(function (path) { return [
                        i
                    ].concat(new Array().concat(path)); });
                })
                    .reduce(function (a, b) { return a.concat(b); }, []);
            }
            else {
                return Object.keys(object)
                    .filter(function (k) { return matches(segment, k); })
                    .map(function (key) {
                    return expand(patternArr.slice(1), object[key]).map(function (path) { return [
                        key
                    ].concat(new Array().concat(path)); });
                })
                    .reduce(function (a, b) { return a.concat(b); }, []);
            }
        }
        else {
            // find all keys in object that matches segment
            // and return this keys
            if (object instanceof Array) {
                return object.map(function (_, i) { return i; }).filter(function (i) { return matches(segment, i); });
            }
            else {
                return Object.keys(object).filter(function (k) { return matches(segment, k); });
            }
        }
    }
}
exports.default = expand;
/**
 * Checks if an pattern segment matches a path segment
 * @param patternSegment the pattern segment
 * @param key the path segment to test against
 *
 * @private
 * @hidden
 */
function matches(patternSegment, key) {
    if (patternSegment instanceof RegExp) {
        return patternSegment.test(String(key));
    }
    else if (patternSegment === "*") {
        return true;
    }
    else {
        return patternSegment === key;
    }
}
//# sourceMappingURL=expand.js.map